@@ -2,15 +2,20 @@
 """ Standard Linear regression models. """
 import warnings
 from cmath import sqrt
+from pathlib import Path
 from pprint import pformat
-from typing import Any, List, Optional, Sequence, Tuple, Union
+from typing import Any, Optional, Sequence, Tuple, Union
 
 import numpy as np
+import pandas as pd
 from brainspace.mesh.mesh_elements import get_cells, get_points
 from brainspace.vtk_interface.wrappers.data_object import BSPolyData
+from nibabel.nifti1 import Nifti1Image
 
 from brainstat._typing import ArrayLike
 from brainstat._utils import deprecated
+from brainstat.datasets import fetch_parcellation, fetch_template_surface
+from brainstat.datasets.base import fetch_template_surface, fetch_yeo_networks_metadata
 from brainstat.mesh.utils import _mask_edges, mesh_edges
 from brainstat.stats.terms import FixedEffect, MixedEffect
 from brainstat.stats.utils import apply_mask, undo_mask
@@ -27,16 +32,16 @@ class SLM:
     def __init__(
         self,
         model: Union[FixedEffect, MixedEffect],
-        contrast: Union[ArrayLike, FixedEffect],
-        surf: Optional[Union[dict, BSPolyData]] = None,
+        contrast: ArrayLike,
+        surf: Optional[Union[str, dict, BSPolyData]] = None,
         mask: Optional[ArrayLike] = None,
         *,
         correction: Optional[Union[str, Sequence[str]]] = None,
-        niter: int = 1,
         thetalim: float = 0.01,
         drlim: float = 0.1,
         two_tailed: bool = True,
         cluster_threshold: float = 0.001,
+        data_dir: Optional[Union[str, Path]] = None
     ) -> None:
         """Constructor for the SLM class.
 
@@ -44,12 +49,18 @@ class SLM:
         ----------
         model : brainstat.stats.terms.FixedEffect, brainstat.stats.terms.MixedEffect
             The linear model to be fitted of dimensions (observations, predictors).
-        contrast : array-like, brainstat.stats.terms.FixedEffect
+            Note that, for volumetric input, BrainStat follows Fortran (MATLAB)
+            convention for ordering voxels, i.e. the first dimension changes
+            first.
+        contrast : array-like
             Vector of contrasts in the observations.
-        surf : dict, BSPolyData, optional
+        surf : str, dict, BSPolyData, Nifti1Image, optional
             A surface provided as either a dictionary with keys 'tri' for its
             faces (n-by-3 array) and 'coord' for its coordinates (3-by-n array),
-            or as a BrainSpace BSPolyData object by default None.
+            or as a BrainSpace BSPolyData object, a string containing a template
+            name accepted by fetch_template_surface, or a Nifti1Image wherein 0
+            denotes excluded voxels and any other value denotes included voxels,
+            by default None.
         mask : array-like, optional
             A mask containing True for vertices to include in the analysis, by
             default None.
@@ -58,9 +69,6 @@ class SLM:
             theory multiple comparisons correction will be run. If it contains
             "fdr" a false discovery rate multiple comparisons correction will be
             run. Both may be provided. By default None.
-        niter : int, optional
-            Number of iterations of the Fisher scoring algorithm for fitting
-            mixed effects models, by default 1.
         thetalim : float, optional
             Lower limit on variance coefficients in standard deviations, by
             default 0.01.
@@ -73,20 +81,29 @@ class SLM:
         cluster_threshold : float, optional
             P-value threshold or statistic threshold for defining clusters in
             random field theory, by default 0.001.
+        data_dir : str, pathlib.Path, optional
+            Path to the location to store BrainStat data files, defaults to
+            $HOME_DIR/brainstat_data.
         """
         # Input arguments.
         self.model = model
-        self.contrast = contrast
-        self.surf = surf
+        self.contrast = np.array(contrast)
+
+        if isinstance(surf, str):
+            self.surf_name = surf
+            self.surf = fetch_template_surface(self.surf_name)
+        else:
+            self.surf_name = None
+            self.surf = surf
+
         self.mask = mask
-        self.correction = correction
-        if isinstance(self.correction, str):
-            self.correction = [self.correction]
-        self.niter = niter
+        self.correction = [correction] if isinstance(correction, str) else correction
+        self.niter = 1
         self.thetalim = thetalim
         self.drlim = drlim
         self.two_tailed = two_tailed
         self.cluster_threshold = cluster_threshold
+        self.data_dir = data_dir
 
         # Error check
         if self.surf is None:
@@ -111,11 +128,17 @@ class SLM:
             An error will be thrown when multivariate data is provided and a
             one-tailed test is requested.
         """
-        if Y.ndim > 2:
+        if Y.ndim < 2 or Y.ndim > 3:
+            raise ValueError("Input data must be two or three dimensional.")
+        elif Y.ndim == 3:
             if (not self.two_tailed) and Y.shape[2] > 1:
-                raise ValueError(
+                raise NotImplementedError(
                     "One-tailed tests are not implemented for multivariate data."
                 )
+            if Y.shape[2] > 3 and "rft" in self.correction:
+                raise NotImplementedError(
+                    "Random Field Theory corrections are not implemented for more than three variates."
+                )
             student_t_test = Y.shape[2] == 1
         else:
             student_t_test = True
@@ -145,8 +168,18 @@ class SLM:
             self.Q = _merge_fdr(Q1, Q2)
         else:
             self.P = P1
+            # Make variable type consistent with two one tailed outputs.
+            for field in ["peak", "clus", "clusid"]:
+                if self.P is not None and field in self.P:
+                    if field == "clusid":
+                        self.P[field] = [self.P[field]]
+                    else:
+                        for field2 in self.P[field].keys():
+                            self.P[field][field2] = [self.P[field][field2]]
             self.Q = Q1
 
+        self._surfstat_to_brainstat_rft()
+
     def _run_multiple_comparisons(self) -> Tuple[Optional[dict], Optional[np.ndarray]]:
         """Runs the multiple comparisons tests and returns their outputs.
 
@@ -180,8 +213,6 @@ class SLM:
         self.r = None
         self.dr = None
         self.resl = None
-        self.tri = None
-        self.lat = None
         self.c = None
         self.ef = None
         self.sd = None
@@ -190,6 +221,64 @@ class SLM:
         self.Q = None
         self.du = None
 
+    def _surfstat_to_brainstat_rft(self) -> None:
+        """Convert SurfStat RFT output to BrainStat RFT output.
+        In short, we convert the peak/clus arrays to pandas dataframes and add Yeo networks.
+        """
+        if self.P is not None:
+            if "peak" in self.P:
+                for i in range(len(self.P["peak"]["t"])):
+                    if self.P["peak"]["vertid"][i] is not None:
+                        self.P["peak"]["vertid"][i] = self.P["peak"]["vertid"][
+                            i
+                        ].astype(int)
+                    if self.P["peak"]["clusid"][i] is not None:
+                        self.P["peak"]["clusid"][i] = self.P["peak"]["clusid"][
+                            i
+                        ].astype(int)
+
+                    if self.surf_name in (
+                        "fsaverage",
+                        "fsaverage5",
+                        "fslr32k",
+                        "civet41k",
+                        "civet164k",
+                    ):
+                        yeo7 = fetch_parcellation(
+                            self.surf_name, "yeo", 7, data_dir=self.data_dir
+                        )
+                        yeo_names, _ = fetch_yeo_networks_metadata(7)
+                        yeo_names.insert(0, "Undefined")
+                        yeo7_index = yeo7[self.P["peak"]["vertid"][i]]
+                        if "yeo7" not in self.P["peak"]:
+                            self.P["peak"]["yeo7"] = []
+                        self.P["peak"]["yeo7"].append(np.take(yeo_names, yeo7_index))
+
+            if "clus" in self.P:
+                for i in range(len(self.P["clus"]["clusid"])):
+                    if self.P["clus"]["clusid"][i] is not None:
+                        self.P["clus"]["clusid"][i] = self.P["clus"]["clusid"][
+                            i
+                        ].astype(int)
+
+            none_squeeze = lambda x: np.squeeze(x) if x is not None else None
+            for field in ["peak", "clus"]:
+                P_tmp = []
+                for i in range(len(self.P[field]["P"])):
+                    tail_dict = {
+                        key: none_squeeze(value[i])
+                        for key, value in self.P[field].items()
+                    }
+                    if tail_dict["P"] is not None:
+                        if tail_dict["P"].size == 1:
+                            P_tmp.append(pd.DataFrame.from_dict([tail_dict]))
+                        else:
+                            P_tmp.append(pd.DataFrame.from_dict(tail_dict))
+                            P_tmp[i].sort_values(by="P", ascending=True)
+                    else:
+                        P_tmp.append(pd.DataFrame(columns=tail_dict.keys()))
+                self.P[field] = P_tmp
+
     def _unmask(self) -> None:
         """Changes all masked parameters to their input dimensions."""
         simple_unmask_parameters = ["t", "coef", "SSE", "r", "ef", "sd", "dfs"]
@@ -213,7 +302,7 @@ class SLM:
     """ Property specifications. """
 
     @property
-    def surf(self) -> Union[BSPolyData, dict, None]:
+    def surf(self) -> Union[BSPolyData, dict, Nifti1Image, None]:
         return self._surf
 
     @surf.setter
@@ -223,6 +312,8 @@ class SLM:
             if isinstance(self.surf, BSPolyData):
                 self.tri = np.array(get_cells(self.surf)) + 1
                 self.coord = np.array(get_points(self.surf)).T
+            elif isinstance(self.surf, Nifti1Image):
+                self.lat = self.surf.get_fdata() != 0
             else:
                 if "tri" in value:
                     self.tri = value["tri"]
@@ -237,7 +328,10 @@ class SLM:
 
     @property
     def tri(self) -> np.ndarray:
-        return self._tri
+        if hasattr(self, "_tri"):
+            return self._tri
+        else:
+            return None
 
     @tri.setter
     def tri(self, value):
@@ -251,7 +345,10 @@ class SLM:
 
     @property
     def lat(self) -> np.ndarray:
-        return self._lat
+        if hasattr(self, "_lat"):
+            return self._lat
+        else:
+            return None
 
     @lat.setter
     def lat(self, value):
@@ -305,13 +402,13 @@ def _merge_rft(P1: dict, P2: dict) -> dict:
         return None
 
     P = {}
-    for key1 in P1:
+    for key1 in P1 or P2:
         P[key1] = {}
         if key1 == "clusid":
             P[key1] = [P1[key1], P2[key1]]
             continue
         for key2 in P1[key1]:
-            if key2 == "P" and key1 == "pval":
+            if key1 == "pval":
                 P[key1][key2] = _onetailed_to_twotailed(P1[key1][key2], P2[key1][key2])
             else:
                 P[key1][key2] = [P1[key1][key2], P2[key1][key2]]
@@ -340,6 +437,13 @@ def _merge_fdr(Q1: Optional[ArrayLike], Q2: Optional[ArrayLike]) -> np.ndarray:
 
 def _onetailed_to_twotailed(p1: ArrayLike, p2: ArrayLike) -> np.ndarray:
     """Converts two one-tailed tests to a two-tailed test"""
+    if p1 is None and p2 is None:
+        return None
+    elif p1 is None:
+        p1 = p2
+    elif p2 is None:
+        p2 = p1
+
     return np.minimum(np.minimum(p1, p2) * 2, 1)
 
 
@@ -422,7 +526,7 @@ def f_test(slm1: SLM, slm2: SLM) -> SLM:
             a22 = SSE2[0, :] * h[2, :] - SSE2[1, :] * h[1, :]
             a0 = a11 * a22 - a12 * a21
             a1 = (a11 + a22) / 2
-            s1 = np.array([sqrt(x) for x in (a1 ** 2 - a0)]).real
+            s1 = np.array([sqrt(x) for x in (a1**2 - a0)]).real
             d = (df2 / (df1 - df2)) / (det + (det <= 0)) * (det > 0)
             slm.t[0, :] = (a1 + s1) * d
             if l == 2:
@@ -455,9 +559,9 @@ def f_test(slm1: SLM, slm2: SLM) -> SLM:
             )
             a1 = a22 * a33 - a23 * a32 + a11 * a33 - a13 * a31 + a11 * a22 - a12 * a21
             a2 = -(a11 + a22 + a33)
-            q = a1 / 3 - a2 ** 2 / 9
-            r = (a1 * a2 - 3 * a0) / 6 - a2 ** 3 / 27
-            s1 = (r + [sqrt(x) for x in (q ** 3 + r ** 2)]) ** (1 / 3)
+            q = a1 / 3 - a2**2 / 9
+            r = (a1 * a2 - 3 * a0) / 6 - a2**3 / 27
+            s1 = (r + [sqrt(x) for x in (q**3 + r**2)]) ** (1 / 3)
             z = np.zeros((3, v))
             z[0, :] = 2 * s1.real - a2 / 3
             z[1, :] = -s1.real - a2 / 3 + np.sqrt(3) * s1.imag
