@@ -229,6 +229,8 @@ def _random_field_theory(self) -> valid_rft_output:
     )[0]
     tlim = tlim[1]
     pval["P"] = pval["P"] * (self.t[0, :] > tlim) + (self.t[0, :] <= tlim)
+    if pval["C"] is not None:
+        pval["C"] = pval["C"].flatten()
 
     return pval, peak, clus, clusid
 
@@ -346,7 +348,7 @@ def stat_threshold(
     if search_volume.shape[1] == 1:
         radius = (search_volume / (4 / 3 * math.pi)) ** (1 / 3)
         search_volume = np.c_[
-            np.ones(radius.shape), 4 * radius, 2 * radius ** 2 * math.pi, search_volume
+            np.ones(radius.shape), 4 * radius, 2 * radius**2 * math.pi, search_volume
         ]
 
     if search_volume.shape[0] == 1:
@@ -556,8 +558,8 @@ def stat_threshold(
             if d == 0:
                 cons = np.log(scale)
             else:
-                cons = (1 - 1 / scale ** d) / d
-            tau[:, d] = rho[:, d, 1] * (1 + 1 / scale ** d) / 2 + s1 * cons
+                cons = (1 - 1 / scale**d) / d
+            tau[:, d] = rho[:, d, 1] * (1 + 1 / scale**d) / 2 + s1 * cons
         rho[:, 0 : D[1], 0] = tau
 
     if D[1] == 0:
@@ -716,7 +718,7 @@ def stat_threshold(
                 print(P_val_extent_1)
     else:
         # Find dbn of S by taking logs then using fft for convolution:
-        ny = 2 ** 12
+        ny = 2**12
         a = d / 2
         b2 = a * 10 * np.max([np.sqrt(2 / (np.min([df1 + df2, np.min(dfw1)]))), 1])
         if df2 < math.inf:
@@ -916,20 +918,37 @@ def peak_clus(
     edg = voxid[edg[np.all(excurset[edg], 1), :]]
     nf = np.arange(1, n + 1)
 
-    # Find cluster id's in nf (from Numerical Recipes in C, page 346):
-    for el in range(1, edg.shape[0] + 1):
-        j = edg[el - 1, 0]
-        k = edg[el - 1, 1]
-        while nf[j - 1] != j:
-            j = nf[j - 1]
-        while nf[k - 1] != k:
-            k = nf[k - 1]
-        if j != k:
-            nf[j - 1] = k
+    # Find cluster id's in nf
+    not_used_indexes = set(np.arange(1, n + 1))
 
-    for j in range(1, n + 1):
-        while nf[j - 1] != nf[nf[j - 1] - 1]:
-            nf[j - 1] = nf[nf[j - 1] - 1]
+    cluster_val = 1
+    for i in range(1, n + 1):
+        if i not in not_used_indexes:
+            continue
+        in_cluster = set([i])
+        not_used_indexes.remove(i)
+
+        neighbours = (
+            set(edg[np.isin(edg, list(in_cluster)).any(axis=1)].ravel())
+            & not_used_indexes
+        )
+        in_cluster = in_cluster | neighbours
+
+        while True:
+            neighbours = (
+                set(edg[np.isin(edg, list(neighbours)).any(axis=1)].ravel())
+                & not_used_indexes
+            )
+            not_used_indexes = not_used_indexes - neighbours
+            if len(neighbours) == 0:
+                break
+            else:
+                in_cluster = in_cluster | neighbours
+
+        in_cluster_idx = [x - 1 for x in list(in_cluster)]
+        nf[in_cluster_idx] = cluster_val
+
+        cluster_val = cluster_val + 1
 
     vox = np.argwhere(excurset) + 1
     ivox = np.argwhere(np.in1d(vox, lmvox)) + 1
@@ -1366,7 +1385,7 @@ def compute_resels(self) -> Tuple[np.ndarray, np.ndarray, np.ndarray]:
             ue = edg2[:, 0] + m1 * (edg2[:, 1] - 1)
             e = edg2.shape[0]
             ae = np.arange(0, e)
-            if e < 2 ** 31:
+            if e < 2**31:
                 sparsedg = csr_matrix(
                     (ae, (ue, np.zeros(ue.shape, dtype=int))), dtype=np.int
                 )
@@ -1393,7 +1412,7 @@ def compute_resels(self) -> Tuple[np.ndarray, np.ndarray, np.ndarray]:
                     lkc1[1, 2] = 0
                     lkc1[2, 2] = 0
                 else:
-                    if e < 2 ** 31:
+                    if e < 2**31:
                         l12 = self.resl[
                             sparsedg[
                                 tri[masktri, 0] + m1 * (tri[masktri, 1] - 1), 0
@@ -1471,7 +1490,7 @@ def compute_resels(self) -> Tuple[np.ndarray, np.ndarray, np.ndarray]:
             masktet = np.all(mask1[tet], axis=1).flatten()
             lkc1[0, 3] = np.sum(masktet)
             if self.resl is not None and k < (K - 1):
-                if e < 2 ** 31:
+                if e < 2**31:
                     l12 = self.resl[
                         (
                             sparsedg[
@@ -1709,6 +1728,7 @@ def compute_resels(self) -> Tuple[np.ndarray, np.ndarray, np.ndarray]:
     # Compute resels - RV
     D1 = lkc.shape[0] - 1
     D2 = lkc.shape[1] - 1
+
     tpltz = toeplitz((-1) ** (np.arange(0, D1 + 1)), (-1) ** (np.arange(0, D2 + 1)))
     lkcs = np.sum(tpltz * lkc, axis=1).T
     lkcs = np.trim_zeros(lkcs, trim="b")
