@@ -1,8 +1,12 @@
 """Unit tests of mesh_edges."""
 
 import pickle
+import sys
 
+import nibabel as nib
 import numpy as np
+import pytest
+import templateflow.api as tflow
 
 from brainstat.mesh.utils import mesh_edges
 
@@ -65,3 +69,16 @@ def test_04():
     infile = datadir("xstatedg_04_IN.pkl")
     expfile = datadir("xstatedg_04_OUT.pkl")
     dummy_test(infile, expfile)
+
+
+@pytest.mark.skipif(
+    sys.platform.startswith("win"), reason="Template flow has issues on windows."
+)
+def test_nifti_input():
+    nifti = nib.load(
+        tflow.get("MNI152Lin", resolution="02", desc="brain", suffix="mask")
+    )
+    edg = mesh_edges(nifti)
+
+    assert edg.shape[1] == 2
+    assert np.amax(edg) <= nifti.get_data().sum() - 1
